function axishandle = showPolicyMap (board, policy, title)
% This function creates a pretty figure of a given policy

    if (nargin<3) | isempty(title)
       title = 'Policy';
    end

    % If there is no figure open, then create one
    if isempty(gcbf)
      fig = figure('name',title);
      set(fig,'Position',[50 50 500 500]);
      set(fig,'Units','Normalized');
      set(fig,'MenuBar','none');
      Caxes2 = axes('position', [0.01 0.01 0.98 0.98],'FontSize',12, ...
        'FontName','Helvetica');
    else
      % get the current figure, and clear it
      gcf; cla;
    end
   
    n = board.n;
    % plot board.field where walls are black, and everything else is white
    walls=board.cost';
    walls(walls < board.wall) = 0;
    pcolor([1:n+1],[1:n+1],[walls walls(:,end); walls(end,:) walls(end,end)]);
    % set the colormap for the ploting the cost and looking really nice
    map = jet;
    cmap = flipud(map);
    cmap(1,:) = zeros(1,3); 
    cmap(2:end,:) = ones(size(cmap,1)-1,3);
    map = flipud(cmap);
    colormap(map);
    % keep the plot so we can plot over it
    hold on;
    % now plot the f values for all tiles evaluated
    data = board.costchart';
  
    axishandle = pcolor([1:n+1],[1:n+1],[data data(:,end); data(end,:) data(end,end)]);
    
    % plot goal as a red circle and start as a green circle
    plot(board.goal.x+0.5,board.goal.y+0.5,'go','MarkerSize',8,'LineWidth',4);
    plot(board.start.x+0.5,board.start.y+0.5,'ro','MarkerSize',8,'LineWidth',4);
    
    for i=1:size(policy,1)
        for j=1:size(policy,2)
	    plot (i+0.5, j+0.5, [policy(i,j),'b'], 'MarkerSize',8);
	end
    end
