function smoothed = smoothedPath(path, alpha, beta)

% Smoothing algorithm (CS373): 
% 1. Initially, create variables, yi, that are the same as the xi. Remember, these are the non-smooth path locations the planner has found: y(i) = x(i) 
% 2. Then you optimize by two criteria by minimizing these two expressions: (x(i) - y(i))^2 and (y(i) - y(i+1))^2
%    The first expression minimizes the distance between the original point and the smooth point, 
%    and the second minimizes the distance between two consecutive smooth points.


% How can you optimize these two terms?
% You can use gradient descent, that is, for every time step, you take a small step in the direction of minimizing the error. 
% The expression for the first objective is
%   yi = yi + alpha * (xi - yi).  
% When you iterate, you assign to yi the old value of yi and add a term that is proportional to the deviation of yi to xi, weighted by the weight function alpha (0.5). 
% The second term you can implement is one where you retain the old y variable, but move a little bit in the direction of yi+1 and away from yi: 
%   yi = yi + beta * (yi+1 - yi) 
% But an even better implementation looks as follows: 
%   yi = yi + beta * (yi+1 + yi-1 - 2 * yi) This combines the step on the left with the step on the right,
% realizing that each  yi occurs twice in the optimization term (yi - yi+1)2. 
% From here, you can implement it in single update rule where you wish  yi to be as close to yi-1 and simultaneously 
% to be as close to yi+1 by optimizing the combined term.
% Set beta to 0.1.

% QUIZ: Alpha -> 0 ==> Straight line (minimization w.r.t. start and end points)

smoothed = path;

tolerance = 0.00001;
change = tolerance;

while (change >= tolerance)

    change = 0.0

    for i=2:size(smoothed,1)-1
        for j=1:size(smoothed,2)

            tmp = smoothed(i,j);
            smoothed(i,j) = smoothed(i,j) + alpha * ( path(i,j) - smoothed(i,j) );
            smoothed(i,j) = smoothed(i,j) + beta * (smoothed(i-1,j) + smoothed(i+1,j) - 2*smoothed(i,j) );
            change = change + abs(tmp-smoothed(i,j));
        end
    end
end

plot(smoothed(:,2)+0.5,smoothed(:,1)+0.5,'Color','g','LineWidth',4);
